package de.fhhannover.inform.trust.ifmapj.binding;

import java.io.InputStream;

import de.fhhannover.inform.trust.ifmapj.exception.EndSessionException;
import de.fhhannover.inform.trust.ifmapj.exception.IfmapErrorResult;
import de.fhhannover.inform.trust.ifmapj.exception.UnmarshalException;
import de.fhhannover.inform.trust.ifmapj.messages.NewSessionResult;
import de.fhhannover.inform.trust.ifmapj.messages.PollResult;
import de.fhhannover.inform.trust.ifmapj.messages.SearchResult;

/**
 * Interface for an {@link Unmarshaller} implementation.
 * 
 * @author aw
 *
 */
public interface Unmarshaller {
	
	/**
	 * Unmarshal the received data to a {@link NewSessionResult}.
	 * 
	 * @param is the response data, i.e. the HTTP body
	 * @return the unmarshalled {@link NewSessionResult}
	 * @throws IfmapErrorResult if the response contains an errorResult
	 * @throws UnmarshalException
	 */
	public NewSessionResult unmarshalNewSessionResult(InputStream is) throws
		IfmapErrorResult, UnmarshalException;

	/**
	 * Unmarshal the received data to a {@link PollResult}.
	 * 
	 * @param is the response data, i.e. the HTTP body
	 * @return a {@link PollResult} containing the search results
	 * @throws IfmapErrorResult if the response contains an errorResult
	 * @throws UnmarshalException
	 * @throws EndSessionException if the session was ended while a poll was pending
	 */
	public PollResult unmarshalPollResult(InputStream is) throws
		IfmapErrorResult, UnmarshalException, EndSessionException;
	
	/**
	 * Unmarshal the received data to a {@link SearchResult}.
	 * 
	 * @param is the response data, i.e. the HTTP body
	 * @return the unmarshalled {@link SearchResult}
	 * @throws IfmapErrorResult if the response contains an errorResult
	 * @throws UnmarshalException
	 */
	public SearchResult unmarshalSearchResult(InputStream is) throws
		IfmapErrorResult, UnmarshalException;
	
	/**
	 * Checks whether the received data contains a endSessionResult element.
	 * 
	 * @param is the response data, i.e. the HTTP body
	 * @throws IfmapErrorResult if the response contains an errorResult
	 * @throws UnmarshalException
	 */
	public void checkEndSessionResult(InputStream is) throws
		IfmapErrorResult, UnmarshalException;
	
	/**
	 * Checks whether the received data contains a renewSessionResult element.
	 * 
	 * @param is the response data, i.e. the HTTP body
	 * @throws IfmapErrorResult if the response contains an errorResult
	 * @throws UnmarshalException
	 */
	public void checkRenewSessionResult(InputStream is) throws
		IfmapErrorResult, UnmarshalException;

	/**
	 * Checks whether the received data contains a purgePublisherReceived element.
	 * 
	 * @param is the response data, i.e. the HTTP body
	 * @throws IfmapErrorResult if the response contains an errorResult
	 * @throws UnmarshalException
	 */
	public void checkPurgePublisherReceived(InputStream is) throws
		IfmapErrorResult, UnmarshalException;

	/**
	 * Checks whether the received data contains a publishReceived element.
	 * 
	 * @param is the response data, i.e. the HTTP body
	 * @throws IfmapErrorResult if the response contains an errorResult
	 * @throws UnmarshalException
	 */
	public void checkPublishReceived(InputStream is) throws
		IfmapErrorResult, UnmarshalException;

	/**
	 * Checks whether the received data contains a subscribeReceived element.
	 * 
	 * @param is the response data, i.e. the HTTP body
	 * @throws IfmapErrorResult if the response contains an errorResult
	 * @throws UnmarshalException
	 */
	public void checkSubscribeReceived(InputStream is) throws
		IfmapErrorResult, UnmarshalException;
}
