package de.fhhannover.inform.trust.ifmapj.identifier;

/**
 * Factory to create different types of {@link Identifier} implementations.
 * 
 * @author aw
 */
public interface IdentifierFactory {
	
	/**
	 * Create an access-request identifier. You MUST set the name with
	 * {@link AccessRequest#setName(String)} afterwards! You may optionally set
	 * the administrative-domain with {@link AccessRequest#setAdministrativeDomain(String)}
	 * too (default is left out).
	 * 
	 * @return the new {@link AccessRequest} instance
	 */
	public AccessRequest createAr();

	
	/**
	 * Create an access-request identifier with the given name. You may optionally
	 * set the administrative-domain with {@link AccessRequest#setAdministrativeDomain(String)}
	 * too (default is left out).
	 * 
	 * @param name the name of the access-request
	 * @return the new {@link AccessRequest} instance
	 */
	public AccessRequest createAr(String name);

	
	/**
	 * Create an access-request identifier with the given name and the given
	 * administrative-domain.
	 * 
	 * @param name the name of the access-request
	 * @param admDom the administrative-domain of the access-request
	 * @return the new {@link AccessRequest} instance
	 */
	public AccessRequest createAr(String name, String admDom);

	
	/**
	 * Create a device identifier. You MUST set the name with
	 * {@link Device#setName(String)} afterwards!
	 * 
	 * @return the new {@link Device} instance
	 */
	public Device createDev();

	
	/**
	 * Create a device identifier with the given name.
	 * 
	 * @param name the name of the device identifier
	 * @return the new {@link Device} instance
	 */
	public Device createDev(String name);

	
	/**
	 * Create an identity identifier. You MUST set its name and type by using
	 * {@link Identity#setName(String)} and {@link Identity#setType(IdentityType)}
	 * afterwards!
	 * 
	 * @return the new {@link Identity} instance
	 */
	public Identity createIdentity();

	
	/**
	 * Create an identity identifier with the given name and the given type.
	 * 
	 * @param type the type of the identity identifier
	 * @param name the name of the identity identifier
	 * @return the new {@link Identity} instance
	 */
	public Identity createIdentity(IdentityType type, String name);
	
	
	/**
	 * Create an identity identifier with the given parameters.
	 * 
	 * @param type the type of the identity identifier
	 * @param name the name of the identity identifier
	 * @param admDom the administrative-domain of the identity identifier
	 * @return the new {@link Identity} instance
	 */
	public Identity createIdentity(IdentityType type, String name, String admDom);

	
	/**
	 * Create an identity identifier.
	 * <br/><br/>
	 * <b>Note: otherTypeDef != null should imply type {@link IdentityType#other}.</b>
	 * 
	 * @param type the type of the identity identifier
	 * @param name the name of the identity identifier
	 * @param admDom the administrative-domain of the identity identifier
	 * @param otherTypeDef vendor specific {@link String}
	 * @return the new {@link Identity} instance
	 */
	public Identity createIdentity(IdentityType type, String name, String admDom,
			String otherTypeDef);

	/**
	 * Create an identity identifier.
	 * 
	 * <b>Note: The type is set to {@link IdentityType#other} by default.</b>
	 * 
	 * @param name the name of the identity identifier
	 * @param admDom the administrative-domain of the identity identifier
	 * @param otherTypeDef vendor specific {@link String}
	 * @return the new {@link Identity} instance
	 */
	public Identity createIdentity(String name, String admDom, String otherTypeDef);

	
	/**
	 * Create an ip-address identifier for IPv4. You MUST set its value with
	 * {@link IpAddress#setValue(String)} afterwards!
	 * 
	 * @return the new ip-address identifier
	 */
	public IpAddress createIp4();


	/**
	 * Create an ip-address identifier for IPv4 with the given value.
	 * 
	 * @param value a {@link String} that represents a valid IPv4 address
	 * @return the new ip-address identifier
	 */
	public IpAddress createIp4(String value);
	
	/**
	 * Create an ip-address identifier for IPv4 with the given value and the
	 * given administrative-domain.
	 * 
	 * @param value a {@link String} that represents a valid IPv4 address
	 * @param admDom the administrative-domain
	 * @return the new ip-address identifier
	 */
	public IpAddress createIp4(String value, String admDom);

	/**
	 * Create an ip-address identifier for IPv6. You MUST set its value with
	 * {@link IpAddress#setValue(String)} afterwards!
	 * 
	 * @return the new ip-address identifier
	 */
	public IpAddress createIp6();


	/**
	 * Create an ip-address identifier for IPv6 with the given value.
	 * 
	 * @param value a {@link String} that represents a valid IPv6 address
	 * @return the new ip-address identifier
	 */
	public IpAddress createIp6(String value);
	
	/**
	 * Create an ip-address identifier for IPv6 with the given parameters.
	 * 
	 * @param value a {@link String} that represents a valid IPv4 address
	 * @param admDom the administrative-domain
	 * @return the new ip-address identifier
	 */
	public IpAddress createIp6(String value, String admDom);

	
	/**
	 * Create an ip-address identifier with the given parameters.
	 * 
	 * @param type the type of the ip-address identifier
	 * @param value a {@link String} that represents a valid IPv4/6 address
	 * @param admDom the administrative-domain
	 * @return the new ip-address identifier
	 */
	public IpAddress createIp(IpAddressType type, String value, String admDom);

	
	/**
	 * Create a mac-address identifier. You MUST set its value with
	 * {@link MacAddress#setValue(String)} afterwards!
	 * 
	 * @return the new mac-address identifier
	 */
	public MacAddress createMac();

	
	/**
	 * Create a mac-address identifier with the given value.
	 * 
	 * @param value a {@link String} that represents a valid MAC address (lower case)
	 * @return the new mac-address identifier
	 */
	public MacAddress createMac(String value);
	
	
	/**
	 * Create a mac-address identifier with the given parameters.
	 * 
	 * @param value a {@link String} that represents a valid MAC address (lower case)
	 * @param admDom the administrative-domain
	 * @return the new mac-address identifier
	 */
	public MacAddress createMac(String value, String admDom);
}
